"""
Snakefile for the paper:

Benchmarking methods for selecting marker genes in single-cell RNA sequencing
data.

Author: Jeffrey Pullin (2020/2021)
Affiliation: University of Melbourne

This snakefile is designed to be run by the run.sh script in this directory.
"""

import json
import re

configfile: "config.yaml"
R = config["R"]

sim_pars = glob_wildcards("config/sim_pars/{sim_par}.json").sim_par
method_pars = glob_wildcards("config/method_pars/{method_par}.json").method_par
all_data_ids = config["all_data_ids"]
general_sim_data_ids = config["general_sim_data_ids"]
time_sim_data_ids = config["time_sim_data_ids"]

# Split the simulation parameters into those used to assess speed and others. 
# Then we can run the speed simulations on only a subset of the dataset, as 
# the simulation parameters are (largely) irrelevant for speed trends. 
time_pattern = re.compile('^num_*')
time_sim_pars = [par for par in sim_pars if time_pattern.match(par)]
general_sim_pars = [par for par in sim_pars if not time_pattern.match(par)]

time_sim_data_combs = expand(
  "{sim_par}-{data_id}", 
  sim_par = time_sim_pars, 
  data_id = time_sim_data_ids
)

general_sim_data_combs = expand(
  "{sim_par}-{data_id}", 
  sim_par = general_sim_pars, 
  data_id = general_sim_data_ids
)

sim_data_combs = time_sim_data_combs + general_sim_data_combs 

data_dirs = expand(
  config["real_data_folder"] + "{data_id}.rds",
  data_id = all_data_ids
)

sim_dirs = [config["sim_data_folder"] + x + ".rds" for x in sim_data_combs]

sim_data_res_dirs = expand(
  config["sim_data_results_folder"] + "{sim_data_comb}-{method_par}.rds", 
  method_par = method_pars,
  sim_data_comb = sim_data_combs
)

# Debugging
# print(sim_dirs)
# print(sim_data_res_dirs)

mg_dirs = expand(
  config["sim_mgs_folder"] + "mg-{sim_data_comb}.rds",
  sim_data_comb = sim_data_combs
)

real_data_res_dirs = expand(
  config["real_data_results_folder"] + "{data_id}-{method_par}.rds", 
  method_par = method_pars,
  data_id = all_data_ids
)

pred_perf_dirs  = expand(
  config["pred_perf_folder"] + "pred_perf-{data_id}-{method_par}-{classifier}.rds", 
  method_par = method_pars,
  data_id = all_data_ids,
  classifier = ["knn", "svm", "sum_max"],
)

summated_perf_dirs = expand(
  "results/summated_perf/summated_perf-{data_id}-{method_par}.rds", 
  method_par = method_pars, 
  data_id = all_data_ids
)

cluster_stability_dirs = expand(
  "results/cluster_stability_sim_data/{method_par}-{n_remove}.rds",
  method_par = method_pars,
  n_remove = list(range(0, 8))
)

cell_stability_dirs = expand(
  "results/cell_stability_sim_data/{method_par}-{percent}.rds",
  method_par = method_pars,
  percent = [100, 75, 50, 25]
)

shell("module load snakemake/7.22.0")
shell("module load R/4.1.2")

rule all:
  input: sim_dirs, sim_data_res_dirs, real_data_res_dirs, mg_dirs, data_dirs,
    pred_perf_dirs, summated_perf_dirs, cluster_stability_dirs, 
    cell_stability_dirs, "data/gene-metrics.rds", "data/cluster-metrics.rds"

rule prepare_data: 
  priority: 102
  input: script = config["code_folder"] + "prep-{data_id}.R",
    raw_data = config["raw_data_folder"] + "{data_id}.rds"
  output: data = config["real_data_folder"] + "{data_id}.rds"
  log: config["logs_folder"] + "prep_data-{data_id}.Rout"
 	resources: 
 	  mem_mb=lambda wildcards, attempt: [8000, 40 * 1000, 150 * 1000][attempt - 1],
	  time_min=lambda wildcards, attempt: [20, 60, 60 * 48][attempt - 1] 
	shell: '''R CMD BATCH --no-restore --no-save\
		"--args raw_data={input.raw_data} data={output.data}"\
		{input.script} {log}'''
		
rule run_method_real_data:
	priority: 101
	input:	script = config["code_folder"] + "run-method.R",
			data = config["real_data_folder"] + "{data_id}.rds",
			method_pars = config["method_pars_folder"] + "{method_id}.json",
			fun = lambda wc: config["code_folder"] + "run-" + wc.method_id.split('_')[0] + ".R"
	output: res = config["real_data_results_folder"] + "{data_id}-{method_id}.rds"
	log:	config["logs_folder"] + "run_method_real_data-{data_id}-{method_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: [8000, 40 * 1000, 150 * 1000][attempt - 1],
	  time_min=lambda wildcards, attempt: [20, 60, 60 * 48][attempt - 1]
	benchmark: 
	  "benchmarks/{data_id}-{method_id}_benchmark.txt"
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.data} fun={input.fun}\
		meth_pars={input.method_pars} res={output.res}"\
		{input.script} {log}'''
		
rule calculate_predictive_performance:
	input: script = config["code_folder"] + "calculate-predictive-perf.R",
	    result = config["real_data_results_folder"] + "{data_id}-{method_id}.rds",
	output: pred_perf = config["pred_perf_folder"] + "pred_perf-{data_id}-{method_id}-{classifier}.rds"
	log: config["logs_folder"] + "pred_perf-{data_id}-{method_id}-{classifier}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 10000,
	  time_min=lambda wildcards, attempt: [10, 60, 600][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args result={input.result} out={output.pred_perf} wcs={wildcards}"\
		{input.script} {log}'''
		
rule calculate_summated_performance:
	input: script = "code/calculate-summated-perf.R",
	    result = "results/real_data/{data_id}-{method_id}.rds",
	output: summated_perf = "results/summated_perf/summated_perf-{data_id}-{method_id}.rds"
	log:  "logs/summated_perf-{data_id}-{method_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 8000,
	  time_min=lambda wildcards, attempt: [10, 60, 600][attempt - 1]
	shell: '''R CMD BATCH --no-restore --no-save\
		"--args result={input.result} out={output.summated_perf} wcs={wildcards}"\
		{input.script} {log}'''
	
rule simulate_data:
  priority: 100
	input:  script = config["code_folder"] + "simulate-data.R",
			sim_pars = config["sim_pars_folder"] + "{sim_id}.json",
			data = config["real_data_folder"] + "{data_id}.rds"
	output: sim = config["sim_data_folder"] + "{sim_id}-{data_id}.rds"
	log:	config["logs_folder"] + "sim_data-{sim_id}-{data_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 8000,
	  time_min=lambda wildcards, attempt: [30, 60, 600][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={output.sim} data={input.data}\
		sim_pars={input.sim_pars} wcs={wildcards}"\
		{input.script} {log}'''

rule extract_marker_genes:
	input:  script = config["code_folder"] + "extract-marker-genes.R",
	    sim_data = config["sim_data_folder"] + "{sim_id}-{data_id}.rds",
	output: mgs = config["sim_mgs_folder"] + "mg-{sim_id}-{data_id}.rds"
	log:	config["logs_folder"] + "extract_mgs-{sim_id}-{data_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 6000,
	  time_min=lambda wildcards, attempt: [30, 60, 600][attempt - 1]
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.sim_data} mgs={output.mgs}"\
		{input.script} {log}'''
		
rule run_method_simulated_data:
	priority: 99
	input:	script = config["code_folder"] + "run-method.R",
			sim_data = config["sim_data_folder"] + "{sim_id}-{data_id}.rds",
			method_pars = config["method_pars_folder"] + "{method_id}.json",
			fun = lambda wc: config["code_folder"] + "run-" + wc.method_id.split('_')[0] + ".R"
	output: res = config["sim_data_results_folder"] + "{sim_id}-{data_id}-{method_id}.rds"
	log:	config["logs_folder"] + "run_method_simulated_data-{sim_id}-{data_id}-{method_id}.Rout"
	resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 8000,
	  time_min=lambda wildcards, attempt: [20, 60, 60 * 36][attempt - 1]
	benchmark: 
	  "benchmarks/{sim_id}-{data_id}-{method_id}_benchmark.txt"
	shell:	'''R CMD BATCH --no-restore --no-save\
		"--args sim={input.sim_data} fun={input.fun}\
		meth_pars={input.method_pars} res={output.res}"\
		{input.script} {log}'''

rule run_method_cluster_stability: 
  input: script = "code/run-method-cluster-stability.R", 
      method_pars = "config/method_pars/{method_par}.json",
      fun = lambda wc: "code/run-" + wc.method_par.split('_')[0] + ".R",
      n = lambda wc: "config/stability_sim_pars/n_remove_" + wc.n_remove + ".rds"
  output: result = "results/cluster_stability_sim_data/{method_par}-{n_remove}.rds"
  log: "logs/run_method_cluster_stability-{method_par}-{n_remove}.Rout"
  resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 8000,
	  time_min=lambda wildcards, attempt: [20, 60, 60 * 36][attempt - 1]
  shell:	'''R CMD BATCH --no-restore --no-save\
		"--args fun={input.fun} n={input.n} result={output.result} meth_pars={input.method_pars}"\
		{input.script} {log}'''	
		
rule run_method_cell_stability: 
  input: script = "code/run-method-cell-stability.R", 
      method_pars = "config/method_pars/{method_par}.json",
      fun = lambda wc: "code/run-" + wc.method_par.split('_')[0] + ".R",
      percent = lambda wc: "config/stability_sim_pars/percent_cells_" + wc.percent + ".rds"
  output: result = "results/cell_stability_sim_data/{method_par}-{percent}.rds"
  log: "logs/run_method_cell_stability-{method_par}-{percent}.Rout"
  resources: 
	  mem_mb=lambda wildcards, attempt: (attempt ** 2) * 8000,
	  time_min=lambda wildcards, attempt: [20, 60, 60 * 36][attempt - 1]
  shell:	'''R CMD BATCH --no-restore --no-save\
		"--args fun={input.fun} percent={input.percent} result={output.result} meth_pars={input.method_pars}"\
		{input.script} {log}'''	
		
rule calculate_metrics:
  input: script = "code/calculate-metrics.R",
  output: gene_metrics = "data/gene-metrics.rds",
    cluster_metrics = "data/cluster-metrics.rds"
  log: "logs/calculate_metrics.Rout",
  resources: mem_mb = 16000, time_min = 60
  shell: '''R CMD BATCH --no-restore --no-save\
    "--args gene_metrics={output.gene_metrics} cluster_metrics={output.cluster_metrics}"\
    {input.script} {log}'''
		
