
suppressPackageStartupMessages({
  library(dplyr)
  library(SingleCellExperiment)
  library(tools)
  library(caTools)
})

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

data_ids <- file_path_sans_ext(list.files(here::here("data", "real_data")))
gene_metrics_file <- args$gene_metrics
cluster_metrics_file <- args$cluster_metrics

# Gene metrics.
out <- list()
for (i in 1:length(data_ids)) {
  data_id <- data_ids[[i]]
  path <- here::here("data", "real_data", paste0(data_id, ".rds"))
  sce <- readRDS(path)
  log_counts <- logcounts(sce)
  out[[i]] <- tibble(
    data_id = data_id,
    gene = rownames(sce),
    gene_mean = rowMeans(log_counts),
    gene_var = rowVars(log_counts)
  )
}
saveRDS(bind_rows(!!!out), gene_metrics_file)

# Cluster metrics.
out <- list()
for (i in 1:length(data_ids)) {
  data_id <- data_ids[[i]]
  path <- here::here("data", "real_data", paste0(data_id, ".rds"))
  sce <- readRDS(path)

  clusters <- unique(colLabels(sce))
  genes <- rownames(sce)

  log_counts <- logcounts(sce)
  auc_log_counts <- as.matrix(t(log_counts))
  data <- exp(log_counts) - 1

  result <- tibble::tibble()
  for (cluster in clusters) {

    cluster_ind <- colLabels(sce) == cluster
    cluster_data <- data[, cluster_ind]
    non_cluster_data <- data[, !cluster_ind]

    num <- rowMeans(cluster_data) + 1
    denom <- rowMeans(non_cluster_data) + 1
    lfcs <- log(num, base = 2) - log(denom, base = 2)

    cluster_means <- rowMeans(log_counts[, cluster_ind])
    non_cluster_means <- rowMeans(log_counts[, !cluster_ind])

    cluster_vars <- rowVars(log_counts[, cluster_ind])
    non_cluster_vars <- rowVars(log_counts[, !cluster_ind])

    aucs <- colAUC(auc_log_counts, as.numeric(cluster_ind))[1, ]

    ave_sd <- (sqrt(cluster_vars) + sqrt(non_cluster_vars)) / 2
    cohens_ds <- (cluster_means - non_cluster_means) / ave_sd

    cluster_result <- tibble::tibble(
      data_id = data_id,
      gene = genes,
      cluster = cluster,
      lfc = lfcs,
      cluster_mean = cluster_means,
      non_cluster_mean = non_cluster_means,
      cluster_var = cluster_vars,
      non_cluster_var = non_cluster_vars,
      auc = aucs,
      cohens_d = cohens_ds
    )

    result <- dplyr::bind_rows(result, cluster_result)
  }

  out[[i]] <- result
}

saveRDS(bind_rows(!!!out), cluster_metrics_file)
