# Script to calculate the predictive performance of the selected marker gene
# sets.

# Get the passed arguments from the command line via Snakemake.
args <- R.utils::commandArgs(trailingOnly = TRUE, asValues = TRUE)

set.seed(20220502)

suppressPackageStartupMessages({
  library(dplyr)
  library(tidyr)
  library(purrr)
  library(SingleCellExperiment)
})

source(here::here("code", "utils.R"))
source(here::here("code", "analysis-utils.R"))

#' Assess the performance of marker gene sets using a multiclass classifier
#'
#' @param result A data frame which contains information about the returned
#'   marker genes
#' @param classifier The classifier to use can be either `"knn"`, `"svm"` or
#'   `"sum_max"`.
#' @param data_id An id of a dataset methods are run on
#' @param method An id of the method.
#' @param fold_k The number of folds to use for cross validation
#' @param knn_k The number of KNNs to use. (Only relevant if classifier is
#'   `"knn"`).
#'
multiclass_prediction <- function(result,
                                  classifier,
                                  method_id, data_id,
                                  n_genes, fold_k = 5, knn_k = 3) {

  sce <- readRDS(here::here("data", "real_data", paste0(data_id, ".rds")))

  fold_ind_tib <- tibble(n = seq_len(ncol(sce)), label = colLabels(sce)) %>%
    group_by(label) %>%
    mutate(fold = sample(1:fold_k, size = n(), replace = TRUE))

  good_sample <- fold_ind_tib %>%
    summarise(n = length(unique(fold))) %>%
    mutate(good = n == fold_k) %>%
    pull(good) %>%
    all()

  while (!good_sample) {
    fold_ind_tib <- tibble(n = seq_len(ncol(sce)), label = colLabels(sce)) %>%
      group_by(label) %>%
      mutate(fold = sample(1:fold_k, size = n(), replace = TRUE))

    good_sample <- fold_ind_tib %>%
      summarise(n = length(unique(fold))) %>%
      mutate(good = n == fold_k) %>%
      pull(good) %>%
      all()
  }

  fold_ind <- fold_ind_tib %>%
    pull(fold)

  gene_list <- result %>%
    pluck("result") %>%
    group_by(cluster) %>%
    group_split() %>%
    set_names(map_chr(., ~as.character(.x$cluster[[1]]))) %>%
    map(~ get_top_sel_mgs(.x, n = n_genes)) %>%
    map(~ .x$gene)

  genes <- flatten_chr(gene_list)

  split_data <- tibble(
    genes = list(genes),
    gene_list = list(gene_list),
    method_id,
    data_id
  ) %>%
    expand_grid(fold = 1:fold_k) %>%
    rowwise() %>%
    mutate(reduced_data = list(t(logcounts(sce[genes, ])))) %>%
    mutate(train_ind = list(fold_ind != fold)) %>%
    mutate(
      train_data = list(reduced_data[train_ind, ]),
      test_data = list(reduced_data[!train_ind, ])
    ) %>%
    select(-reduced_data)

  if (classifier == "knn") {
   classifier_out <- split_data %>%
     mutate(
       result = list(FNN::knn(
          train_data,
          test_data,
          colLabels(sce)[train_ind],
          k = knn_k))
      ) %>%
      mutate(result = list(
        factor(result, levels = levels(colLabels(sce))))
      )
  } else if (classifier == "svm") {
    classifier_out <- split_data %>% mutate(
      svm = list(e1071::svm(train_data, colLabels(sce)[train_ind]))
    ) %>%
      mutate(result = list(predict(svm, test_data)))
  } else if (classifier == "sum_max") {

    run_sum_classifier <- function(test_data, gene_list) {
      n_clusters <- length(gene_list)
      test_data <- scale(test_data)
      sum_mat <- matrix(NA, ncol = n_clusters, nrow = nrow(test_data))
      colnames(sum_mat) <- names(gene_list)
      for (i in 1:n_clusters) {
        sum_mat[, i] <- rowSums(test_data[ , gene_list[[i]], drop = FALSE])
      }
      pred_ind <- apply(sum_mat, 1, which.max)
      out <- factor(colnames(sum_mat)[pred_ind],
                    levels = levels(colLabels(sce)))
      out
    }

    classifier_out <- split_data %>%
      mutate(result = list(run_sum_classifier(test_data, gene_list)))
  }

  out <- classifier_out %>%
    mutate(
      confusion_mat = list(as.matrix(
        table(
          true = colLabels(sce)[!train_ind],
          prediction = result
        )))
    ) %>%
    mutate(
      accuracy = sum(diag(confusion_mat)) / sum(confusion_mat),
      recall = list(diag(confusion_mat) / rowSums(confusion_mat)),
      precision = list(diag(confusion_mat) / colSums(confusion_mat))
    ) %>%
    # Precision can be NaN if no correct predictions are made. This happens
    # for poor performing methods and small clusters.
    mutate(
      precision = list(ifelse(is.nan(precision), 0, precision))
    ) %>%
    mutate(
      f1_score = list(2 * precision * recall / (precision + recall))
    ) %>%
    # Here, F1 score is only NaN if both precision and recall == 0.
    mutate(
      f1_score = list(ifelse(is.nan(f1_score), 0, f1_score))
    ) %>%
    mutate(
      mean_recall = mean(recall),
      mean_precision = mean(precision),
      mean_f1_score = mean(f1_score)
    ) %>%
    mutate(
      median_recall = median(recall),
      median_precision = median(precision),
      median_f1_score = median(f1_score)
    ) %>%
    ungroup() %>%
    select(-c(train_data, test_data, train_ind, result))

  if (classifier == "svm") {
    out <- select(out, -svm)
  }

  out
}

# Debugging.
print(args)
print(args$wcs)

wcs <- parse_wildcards(args$wcs)

print(wcs)

result <- readRDS(here::here(args$result))

pred_perf <- multiclass_prediction(
  result,
  classifier = wcs$classifier,
  method_id = wcs$method_id,
  data_id = wcs$data_id,
  n_genes = 5,
  fold_k = 5,
  knn_k = 3
)

saveRDS(pred_perf, here::here(args$out))
