plot_expert_anno_heatmap <- function(data, data_id, expert_mgs,
                                     metric = "f1_score", n_top = 5) {

  plot_data <- data %>%
    filter(data_id == !!data_id) %>%
    left_join(expert_mgs, by = "cluster") %>%
    rowwise() %>%
    mutate(sel_mgs = list(get_top_sel_mgs(mgs, n_top)$gene)) %>%
    mutate(
      recall = calculate_recall(sel_mgs, expert_mgs),
      precision = calculate_precision(sel_mgs, expert_mgs),
      f1_score = 2 * recall * precision / (recall + precision)
    ) %>%
    mutate(
      f1_score = if_else(recall == 0 & precision == 0, 0, f1_score)
    ) %>%
    ungroup() %>%
    mutate(
      plot_pars = pars_lookup[pars],
      plot_method = method_lookup[method]
    ) %>%
    mutate(plot_pars = fct_reorder(
      factor(plot_pars), !!sym(metric), .fun = sum)
    ) %>%
    mutate(
      recall = round(recall, 3),
      precision = round(precision, 3),
      f1_score = round(f1_score, 3)
    ) %>%
    select(cluster, plot_pars, recall, precision, f1_score)

  rm(data)

  plot_data %>%
    ggplot(aes(x = cluster, y = plot_pars)) +
    geom_tile(aes(fill = !!sym(metric)), col = "black") +
    scale_fill_gradient(low = "white", high = "forestgreen") +
    theme_bw() +
    labs(
      x = "Cluster",
      y = "Method",
      fill = metric_lookup[metric],
      title = paste0(n_top, " genes selected"),
    ) +
    theme(
      panel.grid.major = element_blank(),
      panel.grid.minor = element_blank(),
      panel.border = element_blank(),
      axis.text.x = element_text(angle = 45, vjust = 0.9, hjust = 0.9)
    )
}

plot_expert_anno_cols <- function(data, data_id, expert_mgs, n_top = 5) {

  data %>%
    filter(data_id == !!data_id) %>%
    left_join(expert_mgs, by = "cluster") %>%
    rowwise() %>%
    mutate(
      sel_mgs = list(get_top_sel_mgs(mgs, n_top)$gene),
      count_correct = length(intersect(expert_mgs, sel_mgs)),
    ) %>%
    ungroup() %>%
    mutate(
      plot_pars = pars_lookup[pars],
      plot_method = method_lookup[method]
    ) %>%
    group_by(plot_pars, plot_method) %>%
    summarise(n_correct = sum(count_correct), .groups = "drop") %>%
    mutate(prop_correct = n_correct / sum(lengths(expert_mgs$expert_mgs))) %>%
    mutate(plot_pars = fct_reorder(factor(plot_pars), prop_correct)) %>%
    ggplot(aes(x = plot_pars, y = prop_correct, fill = plot_method)) +
    geom_col() +
    external_package_fill +
    coord_flip(ylim = c(0, 1)) +
    labs(
      x = "Method",
      y = "Proportion recovered",
      title = paste0(n_top, " genes selected"),
      fill = "Package"
    ) +
    theme_bw()
}

plot_expert_anno_n_cluster <- function(data, data_id, expert_mgs, n_top = 5) {

  n_clusters <- length(unique(data$cluster))

  plot_data <- data %>%
    filter(data_id == !!data_id) %>%
    left_join(expert_mgs, by = "cluster") %>%
    rowwise() %>%
    mutate(
      sel_mgs = list(get_top_sel_mgs(mgs, n_top)$gene),
      count_correct = length(intersect(expert_mgs, sel_mgs)),
    ) %>%
    ungroup() %>%
    mutate(
      plot_pars = pars_lookup[pars],
      plot_method = method_lookup[method]
    ) %>%
    group_by(plot_pars, plot_method, cluster, expert_mgs) %>%
    summarise(n_correct = sum(count_correct), .groups = "drop") %>%
    rowwise() %>%
    mutate(prop_correct = n_correct / length(expert_mgs)) %>%
    ungroup() %>%
    mutate(is_recovered = as.numeric(prop_correct == 1)) %>%
    group_by(plot_pars, plot_method) %>%
    summarise(n_correct_clus = sum(is_recovered), .groups = "drop") %>%
    mutate(plot_pars = fct_reorder(factor(plot_pars), n_correct_clus))

    rm(data)

    ggplot(plot_data, aes(x = plot_pars, y = n_correct_clus, fill = plot_method)) +
      geom_col() +
      scale_y_continuous(breaks = 0:n_clusters) +
      coord_flip(ylim = c(0, n_clusters)) +
      package_fill +
      labs(
        x = "Method",
        y = "Number of clusters annotated",
        fill = "Package"
      ) +
      theme_bw()
}
