# This file declares and then saves the expert-annotated marker genes used in
# the analysis.

suppressMessages({
  library(tibble)
  library(openxlsx)
  library(janitor)
  library(dplyr)
  library(stringr)
})

# SMART-seq3 PBMC

ss3_pbmc_expert_mgs <- tibble(
  cluster = c(
    "NK cells",
    "Naive/Memory CD8 T",
    "Cytotoxic T",
    "Naive B",
    "Dendritic cells",
    "CD14 monocytes",
    "plasmacytoid DC",
    "Plasma cells"
  ),
  expert_mgs = list(
    c("NCAM1", "KLRB1"),
    c("PECAM1"),
    c("GZMB", "GZMA"),
    c("CD27", "IGHM", "IGHD", "IL4R"),
    c("KLF4", "CD1C"),
    c("CD14"),
    c("IL3RA", "TLR7"),
    c("PRDM1", "IRF4")
  )
)

# Zeisel

zeisel_expert_mgs <- tribble(
  ~cluster,  ~expert_mgs,
  "interneurons",       "Pnoc",
  "pyramidal SS",       "Tbr1",
  "pyramidal CA1",     "Spink8",
  "oligodendrocytes",     "Hapln2",
  "microglia",       "Aif1",
  "endothelial-mural",      "Acta2",
  "astrocytes_ependymal",      "Aldoc"
)

# Lawlor

lawlor_expert_mgs <- tribble(
  ~cluster, ~expert_mgs,
  "Beta",         "INS",
  "Stellate",  "COL1A1",
  "Ductal",     "KRT19",
  "Alpha",        "GCG",
  "Acinar",     "PRSS1",
  "Gamma/PP",     "PPY",
  "Delta",        "SST"
)

# pbmc3k

pbmc3k_expert_mgs <- tibble(
  cluster = c(
    "Naive CD4 T",
    "CD14+ Mono",
    "Memory CD4 T",
    "B",
    "CD8 T",
    "FCGR3A+ Mono",
    "NK",
    "DC",
    "Platelet"
  ),
  expert_mgs = list(
    c("IL7R", "CCR7"),
    c("CD14", "LYZ"),
    c("IL7R", "S100A4"),
    "MS4A1",
    "CD8A",
    c("FCGR3A", "MS4A7"),
    c("GNLY", "NKG7"),
    c("FCER1A", "CST3"),
    "PPBP"
  )
)

url <- "https://github.com/Teichlab/celltypist_wiki/raw/main/atlases/Pan_Immune_CellTypist/v2/tables/Basic_celltype_information.xlsx"
immune_atlas <- read.xlsx(url)

blood_expert_mgs <- immune_atlas %>%
  as_tibble() %>%
  clean_names() %>%
  rowwise() %>%
  mutate(expert_mgs = list(str_split(curated_markers, ",") %>%
           unlist() %>%
           str_trim()
         )) %>%
  ungroup() %>%
  dplyr::rename(cluster = low_hierarchy_cell_types) %>%
  select(cluster, expert_mgs)

expert_mg_folder <- here::here("data", "expert_mgs")

saveRDS(
  zeisel_expert_mgs,
  here::here(expert_mg_folder, "zeisel_expert_mgs.rds")
)

saveRDS(
  ss3_pbmc_expert_mgs,
  here::here(expert_mg_folder, "ss3_pbmc_expert_mgs.rds")
)

saveRDS(
  lawlor_expert_mgs,
  here::here(expert_mg_folder, "lawlor_expert_mgs.rds")
)

saveRDS(
  pbmc3k_expert_mgs,
  here::here(expert_mg_folder, "pbmc3k_expert_mgs.rds")
)

saveRDS(
  blood_expert_mgs,
  here::here(expert_mg_folder, "blood_expert_mgs.rds")
)

n_markers <- function(mgs) {
  sum(lengths(mgs$expert_mgs))
}

n_markers(zeisel_expert_mgs)
n_markers(pbmc3k_expert_mgs)
n_markers(lawlor_expert_mgs)
n_markers(ss3_pbmc_expert_mgs)

ren <- readRDS(here::here("data", "real_data", "ren.rds"))
stephenson <- readRDS(here::here("data", "real_data", "stephenson.rds"))
yoshida <- readRDS(here::here("data", "real_data", "yoshida.rds"))
dominguez_conde <- readRDS(here::here("data", "real_data", "dominguez_conde.rds"))

blood_expert_mgs %>%
  filter(cluster %in% unique(c(ren$label, stephenson$label, yoshida$label, dominguez_conde$label))) %>%
  n_markers()

sum(
  n_markers(zeisel_expert_mgs),
  n_markers(pbmc3k_expert_mgs),
  n_markers(lawlor_expert_mgs),
  n_markers(ss3_pbmc_expert_mgs),
  blood_expert_mgs %>%
    filter(cluster %in% unique(c(ren$label, stephenson$label, yoshida$label, dominguez_conde$label))) %>%
    n_markers()
)


