#' Find marker genes using cepo
#'
#' @param sce A SingleCellExperiment object,
#' @param pars A list of parameters passed to the function. The empty list.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_cepo <- function(sce, pars) {

  times <- system.time({
    # minCells must be lower than 20, to account for e.g. the Platelet cluster
    # in the pbmc3k dataset.
    cepo_out <- Cepo::Cepo(
      exprsMat = logcounts(sce),
      cellTypes = colLabels(sce),
      minCells = 2
    )
    marker_genes <- Cepo::topGenes(cepo_out, n = 200, returnValues = TRUE)
  })

  result <- purrr::imap(marker_genes, ~
    tibble::tibble(gene = names(.x), value = .x) |>
      dplyr::mutate(cluster = .y)
  ) |>
    dplyr::bind_rows()

  # Fix column names.
  cluster <- result$cluster

  # Fix for the Paul and CITE-seq datasets.
  cluster <- ifelse(
    substr(cluster, 1, 1) == "X" &
    substr(cluster, 2, 2) %in% as.character(0:9),
    substr(cluster, 2, length(cluster)),
    cluster
  )

  # Fix for other datasets.
  cluster <- purrr::map_chr(cluster,
    ~ switch(
      .x,
      # pbmc3k
      "CD14..Mono" = "CD14+ Mono",
      "FCGR3A..Mono" = "FCGR3A+ Mono",
      "CD8.T" = "CD8 T",
      "Naive.CD4.T" = "Naive CD4 T",
      "Memory.CD4.T" = "Memory CD4 T",
      # Lawlor
      "Gamma.PP" = "Gamma/PP",
      "None.Other" = "None/Other",
      # Endothelial
      "Endothelial...arteriole" = "Endothelial - arteriole",
      "Endothelial...CA4..capillary" = "Endothelial - CA4+ capillary",
      "Endothelial...capillary" = "Endothelial - capillary",
      "Endothelial...inflamed" = "Endothelial - inflamed",
      "Endothelial...peribronchiolar" = "Endothelial - peribronchiolar",
      "Endothelial...venule" = "Endothelial - venule",
      # Zeisel
      "pyramidal.CA1" = "pyramidal CA1",
      "pyramidal.SS" = "pyramidal SS",
      "endothelial.mural" = "endothelial-mural",
      # Mesenchymal
      "HAS1.High.Activated.FB" = "HAS1 High Activated FB",
      "Matrix.FB" = "Matrix FB",
      "MyoFB...Activated" = "MyoFB - Activated",
      "PLIN2..FB" = "PLIN2+ FB",
      "WNT2..FB" = "WNT2+ FB",
      # SMART-seq3 pbmc
      "CD14.monocytes" = "CD14 monocytes",
      "Cytotoxic.T" = "Cytotoxic T",
      "Cytotoxic.T.cells" = "Cytotoxic T cells",
      "Dendritic.cells" = "Dendritic cells",
      "Effector.CD4.T" = "Effector CD4 T",
      "FCGR3A.monocytes" = "FCGR3A monocytes",
      "FCGR3A.NK.cells" = "FCGR3A NK cells",
      "HEK" = "HEK",
      "HEK.cells" = "HEK cells",
      "Memory.B" = "Memory B",
      "Naive.B" = "Naive B",
      "Naive.Memory.CD4.T" = "Naive/Memory CD4 T",
      "Naive.Memory.CD8.T" = "Naive/Memory CD8 T",
      "NK.cells" = "NK cells",
      "Plasma.cells" = "Plasma cells",
      "plasmacytoid.DC" = "plasmacytoid DC",
      "Senescent.CD4.T" = "Senescent CD4 T",
      # Zhao
      "CCR7..CD4" = "CCR7+ CD4",
      "CCR7..CD8" = "CCR7+ CD8",
      "CD11c..mem.B" = "CD11c+ mem B",
      "CD14..Mo" = "CD14+ Mo",
      "CD16..Mo" = "CD16+ Mo",
      "CD1c..DCs" = "CD1c+ DCs",
      "Classical.mem.B" = "Classical mem B",
      "CRCR3..CD8" = "CRCR3+ CD8",
      "CX3CR1..CD8" = "CX3CR1+ CD8",
      "CX3CR1..NK" = "CX3CR1+ NK",
      "CXCR3..CD4" = "CXCR3+ CD4",
      "CXCR6..CD8" = "CXCR6+ CD8",
      "CXCR6..NK" = "CXCR6+ NK",
      "cycling.ASC" = "cycling ASC",
      "MerTK..Mo" = "MerTK+ Mo",
      "Mo.doublet" = "Mo doublet",
      "Naive.B" = "Naive B",
      "NKT.doublet" = "NKT doublet",
      "NKT.remove" = "NKT remove",
      "noncycling.ASC" = "noncycling ASC",
      "TCL1A..naive.B" = "TCL1A+ naive B",
      "ZBTB32..mem.B" = "ZBTB32+ mem B",
      # Blood datasets
      "CD16..NK.cells" = "CD16+ NK cells",
      "Classical.monocytes" = "Classical monocytes",
      "Cycling.immune.mix" = "Cycling immune mix",
      "MAIT.cells" = "MAIT cells",
      "Megakaryocytes.platelets" = "Megakaryocytes/platelets",
      "Memory.B.cells" = "Memory B cells",
      "Non.classical.monocytes" = "Non-classical monocytes",
      "Non.switched.memory.B.cells" = "Non-switched memory B cells",
      "Regulatory.T.cells" = "Regulatory T cells",
      "Switched.memory.B.cells" = "Switched memory B cells",
      "Tcm.Naive.cytotoxic.T.cells" = "Tcm/Naive cytotoxic T cells",
      "Tcm.Naive.helper.T.cells" = "Tcm/Naive helper T cells",
      "Tem.Effector.helper.T.cells" = "Tem/Effector helper T cells",
      "Tem.Temra.cytotoxic.T.cells" = "Tem/Temra cytotoxic T cells",
      "Tem.Temra.helper.T.cells" = "Tem/Temra helper T cells",
      "Tem.Trm.cytotoxic.T.cells" = "Tem/Trm cytotoxic T cells",
      .x
    )
  )

  result$cluster <- cluster

  lfcs <- calculate_log_fc(sce, genes = result$gene, clusters = result$cluster)

  result <- dplyr::mutate(
    result,
    raw_statistic = 0,
    log_fc = lfcs,
    scaled_statistic = 0,
    p_value = 0,
    p_value_adj = 0
  )

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = marker_genes, pars = pars)
}
