#' Find marker genes using limma
#'
#' @param sce A SingleCellExperiment object.
#' @param pars parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_limma <- function(sce, pars) {

  `%>%` <- dplyr::`%>%`

  # TODO: Extend the pairwise case.

  times <- system.time({

    n_groups <- length(unique(colLabels(sce)))
    out <- vector("list", length = n_groups)

    for (j in seq_len(n_groups)) {

      group <- levels(colLabels(sce))[[j]]
      x <- factor(as.numeric(colLabels(sce) == group))

      # Fit the edgeR model.
      dge <- edgeR::DGEList(counts(sce), group = x)

      dge <- edgeR::calcNormFactors(dge)
      design <- model.matrix(~x)

      if (pars$fit_method == "voom") {
        vm <- limma::voom(dge, design = design, plot = FALSE)
        fit <- limma::lmFit(vm, design = design)
        fit <- limma::eBayes(fit)
      } else if (pars$fit_method == "trend") {
        y <- new("EList")
        y$E <- edgeR::cpm(dge, log = TRUE, prior.count = 3)
        fit <- limma::lmFit(y, design = design)
        fit <- limma::eBayes(fit, trend = TRUE, robust = TRUE)
      } else {
        stop("Invalid `fit_method`.", call. = FALSE)
      }
      tt <- limma::topTable(fit, number = Inf, adjust.method = "BH")

      # Process the returned list of tags.
      group_out <- as.data.frame(tt) %>%
        tibble::rownames_to_column("gene") %>%
        dplyr::rename(
          log_fc = logFC,
          p_value = P.Value,
          p_value_adj = adj.P.Val,
          scaled_statistic = t
        ) %>%
        dplyr::select(
          -c(AveExpr, B)
        ) %>%
        dplyr::mutate(cluster = group, raw_statistic = 0) %>%
        tibble::as_tibble()

      out[[j]] <- group_out
    }

    result <- dplyr::bind_rows(!!!out)
  })

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = result, pars = pars)
}
