#' Find marker genes using a linear model's i.e. Student's t-test.
#'
#' @param sce A SingleCellExperiment object
#' @param pars parameters passed to the function.
#'   * covariate: the covariate structure to test against. Currently only
#'   `"two_sample"` i.e a one-vs-rest test.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_lm <- function(sce, pars) {

  times <- system.time({

    n_groups <- length(unique(colLabels(sce)))
    n_genes <- nrow(sce)

    out <- vector("list", length = n_groups)
    for (i in seq_len(n_groups)) {

      group <- levels(colLabels(sce))[[i]]
      x <- factor(as.numeric(colLabels(sce) == group))

      t_stat <- function(X) {
        m <- rowMeans(X)
        n <- ncol(X)
        var <- rowSums((X - m) ^ 2)

        list(m = m, n = n, var = var)
      }

      g_1 <- t_stat(logcounts(sce)[, x == 1])
      g_2 <- t_stat(logcounts(sce)[, x == 0])

      num <- g_1$m - g_2$m
      s2 <- (g_1$var + g_2$var) / (g_1$n + g_2$n - 2)
      denom <-  sqrt(s2 * (1 / g_1$n + 1 / g_2$n))

      scaled_statistic <- num / denom
      scaled_statistic <- ifelse(num == 0, 0, scaled_statistic)

      p_value <- pt(abs(scaled_statistic), df = g_1$n + g_2$n - 2,
                    lower.tail = FALSE)
      p_value_adj <- p_value / n_genes

      group_out <- tibble::tibble(
        raw_statistic = num,
        scaled_statistic = scaled_statistic,
        p_value = p_value,
        p_value_adj = p_value_adj,
        log_fc = rep(0, n_genes),
        gene = rownames(sce),
        cluster = group
      )

      # This is implicitly Student's t-test so the degrees of freedom
      # will be same between all genes so sorting on the scaled statistic
      # i.e. the t-value is okay.
      group_out <- dplyr::arrange(group_out, desc(abs(scaled_statistic)))
      out[[i]] <- group_out
    }

    # Group the results for each group together, preserving the within group
    # ordering.
    result <- dplyr::bind_rows(!!!out)
  })

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = result, pars = pars)
}
