#' Find marker genes by ranking based on differences in summaries of lfc.
#'
#' @param sce A SingleCellExperiment object.
#' @param pars parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_logfc <- function(sce, pars) {

  stopifnot(pars$rankby %in% c("raw", "abs"))

  times <- system.time({

    n_groups <- length(unique(colLabels(sce)))

    out <- vector("list", length = n_groups)
    for (i in seq_len(n_groups)) {

      group <- levels(colLabels(sce))[[i]]

      cluster <- sce[, colLabels(sce) == group]
      rest <- sce[, colLabels(sce) != group]

      # This implements Seurat's version of the log fold change.
      cluster_means <- rowMeans(expm1(logcounts(cluster)))
      rest_means <- rowMeans(expm1(logcounts(rest)))

      lfcs <- log(cluster_means + 1, base = 2) - log(rest_means + 1, base = 2)

      group_df <- tibble::tibble(
        log_fc = lfcs,
        raw_statistic = lfcs,
        p_value = 0,
        scaled_statistic = 0,
        p_value_adj = 0,
        gene = rownames(sce),
        cluster = group
      )

      if (pars$rankby == "abs") {
        group_df <- dplyr::arrange(group_df, desc(abs(log_fc)))
      } else if (pars$rankby == "raw") {
        group_df <- dplyr::arrange(group_df, desc(log_fc))
      }

      out[[i]] <- group_df
    }

    result <- dplyr::bind_rows(!!!out)

  })

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = result, pars = pars)
}
