#' Find marker genes using NSforest
#'
#' @param sce A SingleCellExperiment object,
#' @param pars A list of parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_nsforest <- function(sce, pars) {

  library(reticulate)

  old_wd <- getwd()

  # Needed for the import statement below to work
  nsforest_path <- here::here("NSForest")
  setwd(nsforest_path)

  # Load scanpy (R side).
  sc <- import("scanpy")

  # Clear {splatter} `Params` from metadata which cannot be converted.
  sce@metadata <- list()

  # If we are working on datasets of mouse cells there is a problem: some of the
  # gene names start with numbers and this breaks NSForest, which at one point
  # relies on the gene names being syntactically valid Python identifiers. To
  # circumvent this we add a string to the start of all the genes names and
  # then remove it after processing.

  # Test for the Zeisel dataset
  is_zeisel <- function(sce) {
    all(dim(sce) == c(2000, 3005))
  }

  is_paul <- function(sce) {
    "paul15_clusters" %in% colnames(colData(sce))
  }

  is_astrocyte <- function(sce) {
    "Astro_HYPO" %in% colLabels(sce)
  }

  if (is_zeisel(sce) || is_paul(sce) || is_astrocyte(sce)) {
    rownames(sce) <- paste0("A", rownames(sce))
  }

  # Convert the SingleCellExperiment object to an Annotated Data object.
  # We do not use {zellkonverter} as it didn't seem to work when tested.

  # NSForest *does not work* when logcounts is a sparse matrix.
  adata <- sc$AnnData(
    X   = as.matrix(t(logcounts(sce))),
    obs = as.data.frame(colData(sce)),
    var = as.data.frame(rowData(sce))
  )

  py_run_string("from NSForest_v3 import * ")

  # Taken from the README

  # Runs NS_Forest on scanpy object.
  # The (many) warnings appear to be non-fatal.

  times <- system.time({
    nsforest_out <- py$NS_Forest(adata, clusterLabelcolumnHeader = "label")
  })

  markers <- tibble::tibble(nsforest_out[c("clusterName", "NSForest_Markers")])
  colnames(markers) <- c("cluster", "gene")

  result <- tidyr::unnest(markers, cols = everything())

  # Remove the added letter from both the selected genes and the original SCE.
  if (is_zeisel(sce)  || is_paul(sce) || is_astrocyte(sce)) {
    result <- dplyr::mutate(result, gene = substr(gene, 2, nchar(gene)))
    rownames(sce) <- substr(rownames(sce), 2, nchar(rownames(sce)))
  }

  result <- dplyr::arrange(result, cluster)

  # FIXME: NSForest has a *very* strange tendency to sometimes return marker
  # genes which are not present in the original SingleCellExperiment object...
  # For now we simply consider this a software failure and remove those genes
  # from the returned marker genes.
  result <- dplyr::filter(result, gene %in% rownames(sce))

  lfcs <- calculate_log_fc(sce, genes = result$gene, clusters = result$cluster)

  result <- dplyr::mutate(
    result,
    log_fc = lfcs,
    raw_statistic = 0,
    scaled_statistic = 0,
    p_value = 0,
    p_value_adj = 0
  )

  time <- times[[3]]

  validate_mgs_result(result)

  setwd(old_wd)

  # FIXME: Can we save the raw result in the anndata object?
  list(time = time, result = result, raw_result = nsforest_out, pars = pars)
}
