#' Find marker genes using presto
#'
#' @param sce A SingleCellExperiment object,
#' @param pars A list of parameters passed to the function.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_presto <- function(sce, pars) {

  times <- system.time({
    raw_result <- presto::wilcoxauc(sce, group_by = "label")
  })

  # This approximates the output of presto::top_markers. We do not use the
  # function directly as it gives insufficient output.
  raw_result <- dplyr::group_by(raw_result, group)
  raw_result <- dplyr::arrange(raw_result, desc(auc))
  raw_result <- dplyr::ungroup(raw_result)

  result <- dplyr::select(raw_result, -c(avgExpr, pct_in, pct_out))
  # This centres the AUC ensuring that they can be ranked based on their
  # absolute value.
  result <- dplyr::mutate(result, auc = auc - 1 / 2)
  # We also add a non-informative scaled_statistic column.
  result <- dplyr::mutate(result, scaled_statistic = 0)
  result <- dplyr::rename(
    result,
    gene = feature,
    cluster = group,
    log_fc = logFC,
    p_value = pval,
    p_value_adj = padj,
    raw_statistic = auc,
  )
  result <- tibble::as_tibble(result)

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = raw_result, pars = pars)
}

