#' Find marker genes using scran.
#'
#' The wraps the `scran::findMarkers` function.
#'
#' @param sce A SingleCellExperiment object
#' @param pars parameters passed to the function
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_scran <- function(sce, pars) {

  if (pars$func == "findMarkers") {

  times <- system.time({
    raw_result <- scran::findMarkers(
      sce,
      test.type = pars$test.type,
      pval.type = pars$pval.type)
  })

  result <- lapply(raw_result, function(x) {

    x <- as.data.frame(x)
    x <- tibble::rownames_to_column(x, var = "gene")
    x <- tibble::as_tibble(x)

    if (pars$test.type == "wilcox") {
        raw_statistic_col <- "summary.AUC"
    } else {
        raw_statistic_col <- "summary.logFC"
    }

    if (pars$pval.type == "any") {

      # We keep `Top` - which doesn't fit into any category.
      x <- dplyr::select(
        x,
        p_value = p.value,
        raw_statistic = !!raw_statistic_col,
        p_value_adj = FDR,
        gene = gene,
        top = Top
      )
    } else {
      x <- dplyr::select(
        x,
        p_value = p.value,
        raw_statistic = !!raw_statistic_col,
        p_value_adj = FDR,
        gene = gene
      )
    }

    x <- dplyr::mutate(x, scaled_statistic = 0)

    x
  })

  } else if (pars$func == "scoreMarkers") {

    metric <- pars$metric
    rank_by <- pars$rank_by

    times <- system.time({
      raw_result <- scran::scoreMarkers(sce)
    })

    result <- lapply(raw_result, function(x) {

      x <- as.data.frame(x)
      x <- x[, metric, drop = FALSE]
      x <- tibble::rownames_to_column(x, var = "gene")
      x <- tibble::as_tibble(x)

      x <- dplyr::select(
        x,
        raw_statistic = !!metric,
        gene = gene,
      )

      x <- dplyr::mutate(
        x,
        scaled_statistic = 0,
        p_value = 0,
        p_value_adj = 0
      )

      # All statistics except rank are sorted in descending order.
      if (substr(metric, 1, 4) != "rank") {
        x <- dplyr::arrange(x, dplyr::desc(raw_statistic))
      } else {
        x <- dplyr::arrange(x, raw_statistic)
      }

    })
  }

  cluster_names <- names(result)
  for (i in seq_along(result)) {
    result[[i]] <- dplyr::mutate(result[[i]], cluster = cluster_names[[i]])
  }
  result <- dplyr::bind_rows(!!!result)

  # Add in one-vs-rest log fold changes.
  result$log_fc <- calculate_log_fc(
    sce,
    genes = result$gene,
    clusters = result$cluster
  )

  validate_mgs_result(result)

  # Get the elapsed time.
  time <- times[[3]]

  list(time = time, result = result, raw_result = raw_result, pars = pars)
}
