#' Validate that the marker genes returned by a method are in the correct form
#' after processing
#'
#' This function will error informatively if `result` is not in the correct
#' form.
#'
#' @param result A processed returned marker genes object. This is a `tibble`
#'   which certain constraints described in details.
#'
#' @details A processed returned marker gene object should satisfy the
#' following criteria:
#'
#' * Be of class `tibble::tibble`.
#' * Contain a column `gene` which contains the gene labels as strings.
#' * Contain a column `cluster` which contains cluster label as strings.
#'
#' Note that the contents of both `gene` and `cluster` may be either auto
#' generated in the simulated datasets i.e. `"Gene1"`, `"Group1"` or come from
#' real data i.e. `"PF4"` and `"B"`. In particular the real cluster labels may
#' contain spaces.
#'
#' Because both gene and cluster are columns the data is in tidy long format.
#'
#' It should contain the following columns:
#'
#' * `p_value` which contains the returned p-value for each
#'   gene cluster pair. If a p-value is not returned by the method then this
#'   column should contain all zeros.
#' * `p_value_adj` which contains the returned adjusted
#'   p-value for each gene cluster pair. If not returned should be 0. This
#'   column will be ranked smallest first.
#' * `log_fc` which contains the log fold change between the
#'   cluster of interest and the other clusters returned by each method. This
#'   statistic will be ranked in terms of it's absolute value.
#' * `scaled_statistic` which contains the scaled statistic returned by the
#'   method. For our purposes a scaled statistic is any statistic of the form
#'   b / se(b) where b is an estimated parameter and se(b) denotes it's
#'   standard error. Most commonly this will be a Z score but not
#'   that the t-statistic is also of this form. This statistic will be
#'   ranked in terms of it's *absolute value* largest first.
#' * `raw_statistic` which contains a non-scaled statistic. This statistic
#'   will be ranked on based on it's *absolute value*.
#'
validate_mgs_result <- function(result) {

  # Check class.
  if (!inherits(result, "tbl_df")) {
    stop("`result` must be a tibble.", call. = FALSE)
  }

  # Check column names.
  needed_colnames <- c(
    "gene",
    "cluster",
    "p_value",
    "p_value_adj",
    "log_fc",
    "scaled_statistic",
    "raw_statistic"
  )
  if (!all(needed_colnames %in% colnames(result))) {
    stop("Column names missing or incorrect.", call. = FALSE)
  }

  # Check correct types.
  # FIXME: Add check for cluster being either character *or* a factor.
  character_columns <- c("gene")
  for (col in character_columns) {
    if (!is.character(result[[col]])) {
      stop("Column `", col, "` must be of type character.", call. = FALSE)
    }
  }

  numeric_columns <- c(
    "p_value",
    "p_value_adj",
    "log_fc",
    "scaled_statistic",
    "raw_statistic"
  )
  for (col in numeric_columns) {
    if (!is.numeric(result[[col]])) {
      stop("Column `", col, "` must be of type numeric.", call. = FALSE)
    }
  }

  # Check p-values.
  p_value_columns <- c("p_value", "p_value_adj")
  for (col in p_value_columns) {
    p_values <- result[[col]]
    if (!all(p_values >= 0 & p_values <= 1)) {
      stop("Impossible p-values in column: `", col, "`.", call. = FALSE)
    }
  }

}

#' Calculate log fold change
#'
#' @param sce A SingleCellExperiment object
#' @param genes A vector of gene names
#' @param clusters A vector of cluster names
#'
#' @return A vector of one-vs-rest log_2 fold changes between the cells in the
#'   cluster of interest and those in all other clusters.
#'
#' @details We follow Seurat's calculation of the log fold change.
#'
calculate_log_fc <- function(sce, genes, clusters) {

  stopifnot(is(sce, "SingleCellExperiment"))
  stopifnot(all(genes %in% rownames(sce)))
  # FIXME: Cepo will mangle some clusters names.
  #stopifnot(all(clusters %in% colLabels(sce)))
  stopifnot(length(genes) == length(clusters))

  unique_clusters <- unique(clusters)
  unique_genes <- unique(genes)

  data <- exp(logcounts(sce[unique_genes, ])) - 1

  out <- tibble::tibble(gene = genes, cluster = clusters)

  result <- tibble::tibble()
  for (cluster in unique_clusters) {

    cluster_ind <- colLabels(sce) == cluster
    cluster_data <- data[, cluster_ind]
    non_cluster_data <- data[, !cluster_ind]

    num <- rowMeans(cluster_data) + 1
    denom <- rowMeans(non_cluster_data) + 1

    lfcs <- log(num, base = 2) - log(denom, base = 2)

    cluster_result <- tibble::tibble(
      gene = unique_genes,
      cluster = cluster,
      lfc = lfcs
    )

    result <- dplyr::bind_rows(result, cluster_result)
  }

  out <- dplyr::left_join(out, result, by = c("gene", "cluster"))
  out$lfc
}

# tibble(gene = result$gene, lfcs, cluster = result$cluster) %>%
#   filter(gene == "HLA-DRB1")
#
# test$result %>%
#   select(gene, cluster, log_fc) %>%
#   filter(gene == "HLA-DRB1")

