#' Find marker genes using Venice.
#'
#' Wraps functions from the Venice package.
#'
#' @param sce A SingleCellExperiment object
#' @param pars parameters passed to the function
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_venice <- function(sce, pars) {

  `%>%` <- dplyr::`%>%`

  times <- system.time({

    n_groups <- length(unique(colLabels(sce)))
    out <- vector("list", length = n_groups)

    for (j in seq_len(n_groups)) {

      group <- levels(colLabels(sce))[[j]]
      x <- colLabels(sce) == group

      count_mat <- counts(sce)
      if (!isS4(count_mat)) {
        count_mat <- methods::as(count_mat, "sparseMatrix")
      }

      raw_group_out <- Signac::VeniceMarker(count_mat, x)

      group_out <- raw_group_out %>%
        tibble::as_tibble() %>%
        dplyr::rename(
          gene = Gene.Name,
          log_fc = Log2.fold.change,
        ) %>%
        dplyr::mutate(
          p_value = 10^Log10.p.value,
          p_value_adj = 10^Log10.adjusted.p.value
        ) %>%
        dplyr::mutate(
          cluster = group,
          raw_statistic = log_fc,
          scaled_statistic = 0
        ) %>%
        tibble::as_tibble()

      out[[j]] <- group_out
    }

    result <- dplyr::bind_rows(!!!out)
  })

  # Get the elapsed time.
  time <- times[[3]]

  validate_mgs_result(result)

  list(time = time, result = result, raw_result = result, pars = pars)
}
