#' Convert a SingleCellExperiment object to a Seurat data object
#'
#' This is a wrapper around the `as.Seurat` function.
#'
#' @param sce A SingleCellExperiment object
#'
as_seurat_data <- function(sce) {

  stopifnot(is(sce, "SingleCellExperiment"))

  sce2 <- sce

  # Seurat requires the object to have column names.
  if (is.null(colnames(sce2))) {
    colnames(sce2) <- paste0("cell_", seq_len(ncol(sce2)))
  }

  # DelayedArrays are not supported by Seurat.
  if (is(counts(sce2), "DelayedArray")) {
    counts(sce2) <- as(counts(sce2), "dgCMatrix")
  }

  # logcounts are converted to a normal matrix as they are not sparse.
  if (is(logcounts(sce2), "DelayedArray")) {
    logcounts(sce2) <- as(logcounts(sce2), "Matrix")
  }

  out <- Seurat::as.Seurat(sce2)

  # Fix up simulated + Zeisel data for COSG.
  old_assay_name <- names(out@assays)[[1]]
  out <- switch(
    old_assay_name,
    "RNA" = out,
    "endogenous" = SeuratObject::RenameAssays(out, endogenous = "RNA"),
    "originalexp" = SeuratObject::RenameAssays(out, originalexp = "RNA"),
    stop("Unknown assay name")
  )

  # Add the cluster ids to the Seurat object.
  Seurat::Idents(out) <- colLabels(sce2)

  out
}
