# This file contains code to simulate the single cell RNA seq data sets for use
# in the downstream computational experiments.

suppressMessages({
  library(SingleCellExperiment)
  library(scran)
  library(scater)
  library(splatter)
})

source(here::here("code", "utils.R"))

# Get the passed arguments from the command line via Snakemake.
args <- R.utils::commandArgs(trailingOnly = TRUE, asValues = TRUE)

# Debugging.
print(args)

# Get the simulation parameters.
sim_pars <- jsonlite::fromJSON(args$sim_pars)
wcs <- parse_wildcards(args$wcs)

# Debugging.
print(wcs)

print(sim_pars$seed)

set.seed(sim_pars$seed)

full_sim_data <- readRDS(here::here(args$data))

# Reduce to only one cell type before parameter estimation.
# FIXME: Allow multiple subsets to be specified.
sim_data <- switch(
  wcs$data_id,
  "pbmc3k" = full_sim_data[, colLabels(full_sim_data) == "B"],
  "lawlor" = full_sim_data[, colLabels(full_sim_data) == "Alpha"],
  "zeisel" = full_sim_data[, colLabels(full_sim_data) == "oligodendrocytes"],
  "paul" = full_sim_data[, colLabels(full_sim_data) == "14Mo"],
  "ss3_pbmc" = full_sim_data[, colLabels(full_sim_data) == "Naive B"],
  "endothelial" = full_sim_data[, colLabels(full_sim_data) == "Endothelial - capillary"],
  "astrocyte" = full_sim_data[, colLabels(full_sim_data) == "Astro_HYPO"],
  "citeseq" = full_sim_data[, colLabels(full_sim_data) == 2],
  "mesenchymal" = full_sim_data[, colLabels(full_sim_data) == "Pericyte"],
  "zhao" = full_sim_data[, colLabels(full_sim_data) == "Naive B"],
  "stephenson" = full_sim_data[, colLabels(full_sim_data) == "Classical monocytes"],
  "yoshida" = full_sim_data[, colLabels(full_sim_data) == "Classical monocytes"],
  "dominguez_conde" = full_sim_data[, colLabels(full_sim_data) == "Classical monocytes"],
  "ren" = full_sim_data[, colLabels(full_sim_data) == "Classical monocytes"]
)

# Spatter cannot handle sparse matrices (or DelayedArray objects).
sim_counts <- as.matrix(counts(sim_data))
params <- splatEstimate(sim_counts)

# Debugging.
print(params)

# We need to re-weight the probabilities as small numerical issues creep in from
# storing the parameters in json and Splatter is strict about checking they
# sum to 1.
group_prob <- fix_prob(sim_pars$group.prob)

params <- setParam(params, "group.prob", group_prob)
params <- setParam(params, "batchCells", sim_pars$batchCells)
params <- setParam(params, "seed", sim_pars$seed)
params <- setParam(params, "de.facLoc", sim_pars$de.facLoc)
params <- setParam(params, "de.facScale", sim_pars$de.facScale)
# Turn off outlier genes to simplify the selection of true marker genes.
params <- setParam(params, "out.prob", 0)
# Restrict the number of genes in the simulated datasets to not destroy the cluster...
params <- setParam(params, "nGenes", 2000)

if (!is.null(sim_pars$mean.shape)) {
  params <- setParam(params, "mean.shape", sim_pars$mean.shape)
}

if (!is.null(sim_pars$mean.rate)) {
  params <- setParam(params, "mean.rate", sim_pars$mean.shape)
}

Sys.time()

# Run the splatter simulation.
sim <- splatSimulateGroups(params)

Sys.time()

# Remove genes with total count less than 3.
low_gene_inds <- which(rowSums(counts(sim)) <= 3)
if (length(low_gene_inds) > 0) {
  sim <- sim[-low_gene_inds, ]
}

# Construct the log normalized counts needed for downstream analysis.
# TODO: Should we use deconvolution or just the naive size factors?
clusters <- quickCluster(sim)
sim <- computeSumFactors(sim, clusters = clusters)
sim <- logNormCounts(sim)

Sys.time()

# Perform dimensionality reduction.
sim <- runPCA(sim)
sim <- runTSNE(sim, dimred = "PCA")
sim <- runUMAP(sim, dimred = "PCA")

Sys.time()

# Label the columns with the true simulated groups.
colLabels(sim) <- colData(sim)$Group

# Debugging.
print(sim)

saveRDS(sim, args$sim)
