# Produce the simulation parameters for all simulations.
config <- yaml::read_yaml(here::here("config.yaml"))

suppressMessages({
  library(tibble)
  library(tidyr)
  library(dplyr)
  library(purrr)
})

o <- options(scipen = 10)
on.exit(options(scipen = o$scipen), add = TRUE)

# A 'standard' simulation meant to reflect a most plausible scenario.
standard_sims <- expand_grid(
  label = "standard",
  de.facLoc = 3,
  de.facScale = 0.2,
  rep = 1:3,
  batchCells = 2000,
  n_clus = 5,
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column %>%
  mutate(
    seed = 1000 + rowid,
    file_name = paste0(label, "_sim_", rep)
  )

# Changing total number of cells.
num_cells_sims <- expand_grid(
  label = "num_cells",
  de.facLoc = 3,
  de.facScale = 0.2,
  rep = 1:3,
  n_clus = 5,
  batchCells = c(1000, 25000, 50000, 75000, 100000),
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column() %>%
  mutate(
    seed = 3000 + rowid,
    file_name = paste0(label,"_", batchCells, "_sim", "_", rep)
  )

# Changing number of clusters.
num_clusters_sims <- expand_grid(
  label = "num_clusters",
  de.facLoc = 3,
  de.facScale = 0.2,
  n_clus = seq(5, 20, by = 5),
  rep = 1:3,
  batchCells = 20000
) %>%
  rowwise() %>%
  mutate(group.prob = list(rep(1 / n_clus, n_clus))) %>%
  ungroup() %>%
  rowid_to_column() %>%
  mutate(
    seed = 4000 + rowid,
    file_name = paste0(label,"_", n_clus, "_sim", "_", rep)
  )

# Changing de.facLoc parameter.
de_facloc_sims <- expand_grid(
  label = "de_facloc",
  de.facLoc = c(1, 2, 3),
  de.facScale = 0.2,
  rep = 1:3,
  batchCells = 2000,
  n_clus = 5,
  group.prob = list(rep(1 / 5, 5)),
) %>%
  rowid_to_column %>%
  mutate(
    seed = 6000 + rowid,
    file_name = paste0(label, "_", de.facLoc, "_sim_", rep)
  )

all_sim_pars <- list(
  standard_sims,
  num_cells_sims,
  num_clusters_sims,
  de_facloc_sims
)

sim_pars <- flatten(map(all_sim_pars, ~ transpose(.x, .names = .x$file_name)))
sim_ids <- names(sim_pars)

sim_paths <- paste0(here::here(config$sim_pars_folder, sim_ids), ".json")
names(sim_paths) <- sim_ids

# Write all the parameters of each simulation to file.
# The checking of previous parameters is needed so that we don't constantly
# make the files newer, breaking Snakemake.
# FIXME: How can we clean up out of date parameter files?
for (id in sim_ids) {
  new <- sim_pars[[id]]
  path <- sim_paths[[id]]

  if (file.exists(path)) {

    old <- jsonlite::fromJSON(path)
    # Account for small floating point differences.
    if (!isTRUE(all.equal(old, new, tolerance = 1e-2))) {
      write(jsonlite::toJSON(new, null = "null"), path)
    }

  } else {
    write(jsonlite::toJSON(new, null = "null"), path)
  }
}

